#!/usr/bin/python3

import os
import tkinter as tk
from tkinter import filedialog
from tkinter import ttk

import def_classes
import class_plot

def main():
    conf_file=os.getcwd()
    if conf_file[len(conf_file)-1] == '/':
        conf_file=conf_file.strip()+'init_dir.cfg'
    else:
        conf_file=conf_file.strip()+'/init_dir.cfg'

    if not os.path.exists(conf_file.strip()):
        mtext='No such file: '+conf_file+'. Use default directory'
        def_classes.print_message(mtext)
    else:
        frd=open(conf_file,"r")

        fdir=frd.readline()
        fdir=fdir.strip()

        banden=frd.readline()
        banden=banden.strip()

        binen=frd.readline()
        binen=binen.strip()
    
        inout=frd.readline()
        inout=inout.strip()

        ptype=frd.readline()
        ptype=ptype.strip()

        comp=frd.readline()
        comp=comp.strip()

        V0=int(frd.readline())

        comb=frd.readline()
        comb=comb.strip()

        CaXa=frd.readline()
        CaXa=CaXa.strip()

        symmetr=frd.readline()
        symmetr=symmetr.strip()

        frd.close()

    rwd=tk.Tk()
    rwd.title('DSX STATISTICS')
    rwd.geometry('940x760')

    bandL=['High-Band','Mid-Band','Low-Band','All-Bands']
    for i in range(4):
        if banden == bandL[i]:
            break

    tk.Label(rwd,text='Freq Band',bg='white').place(x=12,y=5)
    band=ttk.Combobox(rwd,values=bandL,width=9)
    band.place(x=10, y=25)
    band.current(i)

    tk.Label(rwd,text='B-Width',bg='white').place(x=103,y=5)
    binwd=tk.Entry(rwd, width=5,bg='white')
    binwd.place(x=108,y=25)
    binwd.insert(0,binen)

    if inout == 'In':
        i=0
    elif inout == 'Out':
        i=1
    else:
        i=2

    tk.Label(rwd,text='In_Out',bg='white').place(x=164,y=5)
    ioflag=ttk.Combobox(rwd,values=['In','Out','InOut'],width=5)
    ioflag.place(x=164,y=25)
    ioflag.current(i)

    opList=['Va_CaRaPa','CaRaPa','CaRaPa_d','CaRa_fcefpe','CaRa_comp', \
        'fr+_vs_fr-','Ia+_vs_Ia-','Va+_vs_Va-','Ca_Only','Ra_Only', \
        'Pa_Only','Eff_Only','p_CaRaPa','Q','Ra&Ex','Current','ieee_paper']
    for i in range(13):
        if ptype == opList[i]:
            break

    tk.Label(rwd,text='plot_type',bg='white').place(x=239,y=5)
    plottp=ttk.Combobox(rwd,values=opList,width=10)
    plottp.place(x=230,y=25)
    plottp.current(i)

    if comp == 'Yes':
        i=0
    else:
        i=1

    tk.Label(rwd,text='In_vs_Out',bg='white').place(x=329,y=5)
    compa=ttk.Combobox(rwd,values=['Yes','No'],width=5)
    compa.place(x=335,y=25)
    compa.current(i)

    V0List=[9696,8080,7272,262,16,24,32,40,48,56,64,72,80,88,96,104]
    for i in range(16):
        if V0 == V0List[i]:
            break

    tk.Label(rwd,text='Vdrive',bg='white').place(x=400,y=5)
    V0f=ttk.Combobox(rwd,values=V0List,width=4)
    V0f.place(x=401,y=25)
    V0f.current(i)

    if comb == 'Yes':
        i=0
    else:
        i=1

    tk.Label(rwd,text='Y+ + Y-',bg='white').place(x=458,y=5)
    combin=ttk.Combobox(rwd,values=['Yes','No'],width=4)
    combin.place(x=458,y=25)
    combin.current(i)

    if CaXa == 'Ca':
        i=0
    else:
        i=1

    tk.Label(rwd,text='Ca or Xa',bg='white').place(x=516,y=5)
    corx=ttk.Combobox(rwd,values=['Ca','Xa'],width=4)
    corx.place(x=516,y=25)
    corx.current(i)

    if symmetr == 'Yes':
        i=0
    else:
        i=1

    tk.Label(rwd,text='Symmetric',bg='white').place(x=574,y=5)
    symm=ttk.Combobox(rwd,values=['Yes','No'],width=4)
    symm.place(x=574,y=25)
    symm.current(i)

    #menubar and related actions
    menubar=tk.Menu(rwd)

    # create a pulldown menu, and add it to the menu bar
    filemenu = tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label="File", menu=filemenu)

    helpmenu = tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label="Help", menu=helpmenu)
    helpmenu.add_command(label="About", command=def_classes.about_message)

    rwd.configure(menu=menubar,background='white')

    opendir=def_classes.directory_dialog(fdir)
    filemenu.add_command(label="Select Directory",command=opendir.select_dir)

    saveconfig=def_classes.save_cfg(opendir,conf_file,band,binwd,ioflag,plottp, \
        compa,V0f,combin,corx,symm)
    filemenu.add_command(label="Save Config", command=saveconfig.save_config)

    exitmenu = def_classes.close_button(rwd,opendir,conf_file,band,binwd,ioflag, \
        plottp,compa,V0f,combin,corx,symm)
    filemenu.add_command(label="Exit", command=exitmenu.clean_close)

#plot button for making plots
    wplotbt=class_plot.plot_button(rwd,opendir,band,binwd,ioflag,plottp, \
        compa,V0f,combin,corx,symm)
    pbtn=tk.Button(rwd,text='Plot',width=2,height=2,bg='grey',font=("Courier", 10), \
                   command=wplotbt.wplot)
    pbtn.place(x=840,y=5)

#close button for close the GUI
    wclosebt = def_classes.close_button(rwd,opendir,conf_file,band,binwd,ioflag, \
        plottp,compa,V0f,combin,corx,symm)
    pbtn=tk.Button(rwd,text='Close',width=2,height=2,bg='grey',font=("Courier", 10), \
                  command=wclosebt.clean_close)
    pbtn.place(x=890,y=5)

    rwd.mainloop()

if __name__== "__main__":
    main()
