import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import ant_impedance
import Cs_Rind_Lt
import days_of_year
import binned_values
import moving_average

#####################################################################################
#####################################################################################
#-------------------- plot Ra only for Y + and Y- and Ex for whistler transmission --
#-------------------- paper figure4 -------------------------------------------------
def plot_RaEx_fig4(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    V0, comb):

    print('Read calculated antenna parameters ...')
    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        #if comb == 'Yes':
        #    datafile=fdir+'All-Bands_InOut_data_comb.dat'
        #else:
            datafile=fdir+'All-Bands_InOut_data.dat'
    else:
        #if comb == 'Yes':
        #    datafile=fdir+'/All-Bands_InOut_data_comb.dat'
        #else:
            datafile=fdir+'/All-Bands_InOut_data.dat'

    with open(datafile,'r') as dataf:
        dataf.readline()
        dataf.readline()

        fl=dataf.readlines()

        dataf.close()

    print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,1.95),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=48)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.08,0.53,0.53]
    yB=[0.63,0.30,0.63,0.29]
    wh=[0.44,0.42,0.44,0.44]
    ht=[0.26,0.26,0.26,0.26]

    if comb == 'Yes':
        ndata=1
        nplt=2
    else:
        ndata=2
        nplt=4

    sc=[0]*nplt
    axes=[]

    for i in range(nplt):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(nplt):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=''
    if V0 == 9696:
        sL1=sL1+' V1 >= 96 V'
    elif V0 == 8080:
        sL1=sL1+' V1 >= 80 V'

    if bname == 'All-Bands':
        if comb == 'Yes':
            minRa=0.1
            maxRa=100
            if V0 == 8080:
                minRa=0.1
                maxRa=300
        else:
            minRa=0.03
            maxRa=60
            if V0 == 8080:
                minRa=0.1
                maxRa=50
    elif bname == 'High-Band':
        minRa=0.2
        maxRa=3.0
    elif bname == 'Mid-Band':
        minRa=0.3
        maxRa=40.0

    pi=math.pi
    #pi2=2.0*pi
    LL=[0.0375, 0.3015, 2.1515]

    date=[]
    fres=[[],[]]
    Ra=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()
        datet=a[0]
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 9696 and float(a[2]) < 96.0:
            print('Driving voltage is < ',96)
            diff=diff+1
            continue
        elif V0 == 8080 and float(a[2]) < 80.0:
            print('Driving voltage is < ',80)
            diff=diff+1
            continue

        if bname == 'High-Band':
            if L1 > 8.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Mid-Band':
            if L1 < 0.02*LL[1] or L1 > 5.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Low-Band':
            if L1 < 0.5*LL[2]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpflag != 'InOut':
            if inpflag == 'In': #skip if not in plasmasphere
                if inpl == 0 or inpl == 2 or inpl == 3:
                    print('Out of the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            else:
                if inpl == 1:  #skip if in the plasmasphere
                    print('In the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue

        """
        if comb == 'Yes':
            fqp=float(a[18])
            Ra1=float(a[19])

            if (fqp/1.0e3 > 3.8 and fqp/1.0e3 <6.0):
                diff=diff+1
                continue

            if (fqp > 10.2e3 and Ra1 > 1.e4):
                diff=diff+1
                continue

            fqm=fqp
            Ra2=Ra1
        else:
            fqp=float(a[5])
            fqm=float(a[16])
            Ra1=float(a[9])                        #Ra in Ohm
            Ra2=float(a[20])

            if (fqp > 6.0e3 and Ra1 > 2.5e4) or (fqm > 6.0e3 and Ra2 > 2.5e4):
                diff=diff+1
                continue
        """

        fqp=float(a[5])
        fqm=float(a[16])
        Ra1=float(a[9])                        #Ra in Ohm
        Ra2=float(a[20])

        if(L1 < 0.06 and (fqp>30.1e3 or fqm>30.1e3)):
            diff=diff+1
            continue
        if ((L1 > 0.1 and L1 < 0.5) and (fqp>10.2e3 or fqm>10.2e3)):
            diff=diff+1
            continue
        if (L1 > 1.0 and (fqp > 3.9e3 or fqm >3.9e3)):
            diff=diff+1
            continue

        Ca1=float(a[11])            # antenna Ca in pF
        Ca2=float(a[22])

        Ra1=Ra1+Ra2
        Ca1=Ca1*Ca2/(Ca1+Ca2)

        date.append(datet)

        fres[0].append(fqp/1.0e3)          #frequency in kHz
        Ra[0].append(np.log10(Ra1/1.0e3))  #log scale in kilo Ohm

        fres[1].append(fqm/1.0e3)
        Ra[1].append(np.log10(Ra2/1.0e3))

    #end 'for loop' for calculating antenna Ra

    if diff > 0:
        print(diff,'data out of',nrecd,'not used in analysis')

    # -------- calculate mean values in frequency bins ---------------
    #Cab, Rab, pwb, bcnt, fbin = binned_values.mean_values(bname,bwd,inpflag, \
    #        fres,fsmin,fsmax,Ca,Ra,pwr)

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    elif bname == 'High-Band':
        xscale='linear'
        #yscale='linear'
        fmin=23
        fmax=32
    else:
        xscale='linear'
        #yscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    N=8   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=80 # for 2D binning not for 1D
    y=np.zeros(ybin+1)
    yRa=np.zeros(ybin)

    sc=[0]*3
    for i in range(ndata):
        if i == 0:
            kb=0
        else:
            kb=3

        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        Ramin=min(Ra[i])
        Ramax=max(Ra[i])
        dRa=(Ramax-Ramin)/ybin

        #--- bin size and center of bins for Ra
        for k in range(ybin+1):
            y[k]=Ramin+float(k)*dRa
            if k > 0:
                yRa[k-1]=0.5*(y[k-1]+y[k])
                yRa[k-1]=10.0**yRa[k-1]

        # --- calculate number of Ra data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Ra[i],fres[i],statistic='count', \
            bins=[x,y])
        a=counts.statistic  # number of data points for Ra
        a=a.transpose()

        rdl=len(a[0])+1-N
        Rab2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            Rab2d[k]=moving_average.data_points_running_average(a[k], N)
        Rab2d=Rab2d.transpose()

        # plot number of data points in each bin, onece for yCa (or yRa or ypw)
        # points at given frequency xfr[j]. The data points number is normalized
        # by maximum at each frequency
        for j in range(len(xfr[0:rdl])):
            xx=[xfr[j]]*ybin

            Rabmax=max(Rab2d[j])
            if Rabmax > 0:
                sc[0]=axes[kb].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1, \
                    marker='s')

        #----- set axis properties for Ra ------
        axes[kb].set_xlim(fmin,fmax)
        axes[kb].set_xscale(xscale)
        axes[kb].set_yscale('log')
        axes[kb].set_ylim(minRa,maxRa)
        axes[kb].tick_params(axis='both',which='major',length=1.1,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')

        if bname == 'All-Bands':
            axes[kb].set_xticks([2.5,5,10,20,30,40])
            axes[kb].set_xticklabels(['2.5','5','10','20','30','40'])

    axes[0].set_xlabel('Resonance Frequency (kHz)',fontsize=2.7, \
            labelpad=0.7)
    axes[0].set_ylabel('$R_a$ (Kilo Ohm)',fontsize=2.7,labelpad=0.7)

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' Inside Nominal Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' Outside Plasmasphere'
    else:
        txt=sL1+' All Regions'

    if comb == 'No':
        # make y-axis tick lables invisible for right panels
        for j in range(3,6):
            axes[j].set_yticklabels([])

        xloc=1.0
        axes[0].text(0,1.04,'Y+ 1st Harmonic',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
        axes[3].text(1,1.04,'Y- 1st Harmonic',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[3].transAxes)
    else:
        xloc=0.5
        #for j in range(3):
        #    axes[j].legend(loc='upper center',fontsize=1.6,bbox_to_anchor=(0.65,0.99), \
        #    frameon=False)

    axes[0].text(xloc,1.12,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(xloc,1.03,txt,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)

    # ---- horizontal color bar ----
    """
    if comb == 'Yes':
        xloc=0.085
    else:
        xloc=0.6
    axins = inset_axes(axes[2], width='70%', height='8%', loc='lower left', \
    bbox_to_anchor=(xloc,-0.52,1,1), bbox_transform=axes[2].transAxes)
    cbar=fig.colorbar(sc[2], cax=axins, shrink=0.9, pad=0.04, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=2.5, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.5, labelpad=1.1)
    cbar.outline.set_visible(False)
    """

    # ---- vertical color bar ----
    cbar=fig.colorbar(sc[0], ax=axes[0], shrink=0.9, pad=0.03, aspect=10)
    cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.2, pad=0.4)
    cbar.ax.set_ylabel('Relative No. of Data Points', fontsize=2, labelpad=1.1)
    cbar.outline.set_visible(False)

    # ---- plot theoretical Ra with given fpe, fce and I0 vs f -----
    fpe=400.0e3
    fce=100.0e3
    freq=10.0e3
    d=40.0
    mu0=4.0e-7*pi
    I0=0.05
    c=3.0e8
    nfq=30
    Ram=np.zeros(nfq)
    ff=np.zeros(nfq)
    fmin=2.7
    fmax=33.0
    df=(fmax-fmin)/float(nfq-1)
    coef=3.0e-9*(pi*fce)**3*d*d*mu0*fpe/c
    for i in range(nfq):
        ff[i]=fmin+df*float(i)

        #theoretical Ra in kOhm
        Ram[i]=coef/(ff[i])**2

#   add theoretical Ra to top panel
    axes[0].plot(ff, Ram, lw=0.3, color='blue', label='Theory')

    ff=[2.7,33.0]
    RR=[27,0.39]
    axes[0].plot(ff, RR, ls='dashed', lw=0.3, color='black', label='Fitting')

    axes[0].legend(loc='upper center',fontsize=2,bbox_to_anchor=(0.66,0.99), \
        frameon=False)

    #-------- plot theoretical Ex with given fpe, fce, and f vs r ------------
    fpe=177.0e3
    fce=133.0e3
    I0=0.035
    freq=9.91e3
    r0=10.0e3
    rmax=60000.0e3

    nlen=61
    dr=(rmax - r0)/(nlen-1)
    coef=1.0/np.sqrt(2)*pi*I0*d*mu0*np.sqrt(fce**3*fpe/(c*freq))

    r=np.zeros(nlen)
    Ex=np.zeros(nlen)

    for i in range(nlen):
        r[i]=r0+dr*float(i)
        Ex[i]=coef/np.sqrt(r[i])*1.0e3

    axes[1].plot(r/1.0e3, Ex, lw=0.3, color='black', label='Calculated')
    axes[1].set_xlabel('Radial Distance (km)',fontsize=2.7,labelpad=0.7)
    axes[1].set_ylabel(r'$E_{x}$ (mV/m)',fontsize=2.7,labelpad=0.7)
#major ticks
    axes[1].tick_params(axis='both',which='major',length=1.1,width=0.4, \
        pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
        direction='in')
# set x-axis minor ticks only
    axes[1].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
        pad=1,labelsize=0,left=False,right=False,bottom=True,top=True, \
        direction='in')
    axes[1].set_yscale('log')
    axes[1].set_xscale('log')
    axes[1].set_ylim([1.0e-4,1.0e2])

#   add Arase data range
    rsat=[1117.0, 1117.0]
    Emed=[1.9e-2, 1.6e-1]
    rr=[1087, 1147]
    ExL=[1.9e-2,1.9e-2]
    Exh=[1.6e-1,1.6e-1]
    axes[1].plot(rsat, Emed, lw=0.3, color='blue', label='Arase Measurement Range')
    axes[1].plot(rr, ExL, lw=0.3, color='blue')
    axes[1].plot(rr, Exh, lw=0.3, color='blue')
    axes[1].set_xticks([10,1.0e2,1.0e3,1.0e4,6.0e4])
    axes[1].set_xticklabels(['10',r'$10^2$',r'$10^3$',r'$10^4$',r'$6\times 10^4$'])

    axes[1].legend(loc='upper center',fontsize=2,bbox_to_anchor=(0.75,0.99), \
        frameon=False)

    return canv, fig, toolbar
