import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

from scipy import interpolate

import ant_impedance
#import Cs_Rind_Lt
#import days_of_year
import moving_average

#-------------------- plot Ra, Ca, and Prad only for Y + and Y- ---------------------------
def plot_RaCa_comparison(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    V0, corx):

    print('Read calculated antenna parameters ...')
    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        datafile=fdir+'All-Bands_InOut_data.dat'
    else:
        datafile=fdir+'/All-Bands_InOut_data.dat'

    with open(datafile,'r') as dataf:
        dataf.readline()
        dataf.readline()

        fl=dataf.readlines()

        dataf.close()

    print('Finished reading data')

    fcefpe='/home/jtu/python_code/dsx_fec_fpe_orb/orbit_fce_fpe.dat'

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,1.8),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=48)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.08]
    yB=[0.53,0.15]
    wh=[0.50,0.50]
    ht=[0.36,0.36]

    axes=[]

    for i in range(2):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(2):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=''
    if V0 == 8080:
        sL1=sL1+' V1 >= 80 V'

    """
    if bname == 'All-Bands':
        if corx == 'Ca':
            minCX=10.0
            maxCX=500.0
        else:
            minCX=4
            maxCX=300.0
        minRa=0.2
        maxRa=60
        if V0 == 8080:
            minRa=0.1
            maxRa=40
    elif bname == 'High-Band':
        if corx == 'Ca':
            minCX=400.0
            maxCX=700.0
        else:
            minCX=18
            maxCX=27
        minRa=0.2
        maxRa=3.0
    elif bname == 'Mid-Band':
        if corx == 'Ca':
            minCX=450.0
            maxCX=850.0
        else:
            minCX=18
            maxCX=27
        minRa=0.2
        maxRa=10.0
    elif bname == 'Low-Band':
        if corx == 'Ca':
            minCX=450.0
            maxCX=900.0
        else:
            minCX=18
            maxCX=27
        minRa=0.3
        maxRa=40.0
    """

    pi=math.pi
    pi2=2.0*pi
    LL=[0.0375, 0.3015, 2.1515]

    date=[]
    ut=[]
    fres=[]
    CaXa=[]
    Ra=[]
    CXth=[]
    Rath=[]

    La=41.0
    raa=0.15e-3
    N0=2000.0e6
    qe=1.6022e-19
    eps0=8.85e-12

    fpe=400.0e3
    fce=100.0e3

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 8080 and float(a[2]) < 80.0:
            print('Driving voltage is < ',80)
            diff=diff+1
            continue

        if bname == 'High-Band':
            if L1 > 8.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Mid-Band':
            if L1 < 0.02*LL[1] or L1 > 5.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Low-Band':
            if L1 < 0.5*LL[2]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpflag != 'InOut':
            if inpflag == 'In': #skip if not in plasmasphere
                if inpl == 0 or inpl == 2 or inpl == 3:
                    print('Out of the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            else:
                if inpl == 1:  #skip if in the plasmasphere
                    print('In the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            
        #fqp=float(a[18])
        #Ra1=float(a[19])                        #Ra in Ohm

        #if (fqp > 10.2e3 and Ra1 > 1.0e4):
        #    diff=diff+1
        #    continue

        #Ca1=float(a[20])
        #Ia1=float(a[22])*1.0e-3      # Ia in A

        fqp=float(a[5])
        fqm=float(a[16])
        Ra1=float(a[9])                        #Ra in Ohm
        Ra2=float(a[20])

        if(L1 < 0.06 and (fqp>30.1e3 or fqm>30.1e3)):
            diff=diff+1
            continue
        if ((L1 > 0.1 and L1 < 0.5) and (fqp>10.2e3 or fqm>10.2e3)):
            diff=diff+1
            continue
        if (L1 > 1.0 and (fqp > 3.9e3 or fqm >3.9e3)):
            diff=diff+1
            continue

        Ca1=float(a[11])            # antenna Ca in pF
        Ca2=float(a[22])
        Ia1=float(a[8])*1.0e-3      # Ia in A

        Ra1=Ra1+Ra2
        Ca1=Ca1*Ca2/(Ca1+Ca2)

        date.append(datet)
        ut.append(float(a[1][0:2])*60.0+float(a[1][3:5])+float(a[1][6:8])) #Ut in min

        # measured antenna reactance +/- branches in Ohm
        Xa1=1.0e12/(pi2*fqp*Ca1)

        # theoretical reactance Xa, Ca for each of 3 wires & one element
        Omega1=pi2*fqp
        aa=pi*Omega1*La
        Xs1=0.25/(aa*eps0)*(np.log(Ia1/(aa*qe*N0*raa*raa)+2.0)-1.0)
        Cth1=1.0e12/(Omega1*Xs1)

        #total theoretical Xa, Ca and Ra of three wires connected in parallel
        Xs1=2.0*Xs1/3.0
        Cth1=3.0*Cth1/2.0
        #Va1=float(a[21])
        #Rth1=np.sqrt((Va1/Ia1)**2-Xs1*Xs1)

        if corx == 'Ca':
            CaXa.append(Ca1)
            CXth.append(Cth1)
        else:
            CaXa.append(np.log10(Xa1))
            CXth.append(Xs1)

        Rath.append(3.9e-13*fce**3*fpe*La*La/(fqp*fqp))

        fres.append(fqp/1.0e3)          #frequency in kHz
        Ra.append(np.log10(Ra1))  #log scale in kilo Ohm

    #-----------------------------------------------------------
    #end 'for loop' for input antenna parameters
    #-----------------------------------------------------------

    nf=60
    fmid=6.75e3
    fmide=10.2e3
    Ia1=30e-3
    df=int((fmide-fmid)/(nf-1))+1
    fm=np.zeros(nf)
    Xa_fm=np.zeros(nf)
    Ra_fm=np.zeros(nf)
    for i in range(nf):
        fm[i]=fmid+float(i)*df
        Omega1=pi2*fm[i]
        aa=pi*Omega1*La
        Xa_fm[i]=2.0/3.0*0.25/(aa*eps0)*(np.log(Ia1/(aa*qe*N0*raa*raa)+2.0)-1.0)
        Ra_fm[i]=3.9e-13*fce**3*fpe*La*La/(fm[i]*fm[i])

    nf=60
    fhigh=22.2e3
    fhighe=25e3
    Ia1=200e-3
    df=int((fhighe-fhigh)/(nf-1))+1
    fh=np.zeros(nf)
    Xa_fh=np.zeros(nf)
    Ra_fh=np.zeros(nf)
    for i in range(nf):
        fh[i]=fhigh+float(i)*df
        Omega1=pi2*fh[i]
        aa=pi*Omega1*La
        Xa_fh[i]=2.0/3.0*0.25/(aa*eps0)*(np.log(Ia1/(aa*qe*N0*raa*raa)+2.0)-1.0)
        Ra_fh[i]=3.9e-13*fce**3*fpe*La*La/(fh[i]*fh[i])

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fsmin=min(fres)
    fsmax=max(fres)

    if bname == 'All-Bands':
        xscale='log'
        if corx == 'Ca':
            yscale='linear'
        else:
            yscale='log'
        fmin=2.5
        fmax=40
    elif bname == 'High-Band':
        xscale='linear'
        yscale='linear'
        fmin=23
        fmax=32
    else:
        xscale='linear'
        yscale='linear'
        fmin=min(fres)
        fmax=max(fres)

    # --- calculate dipole antenna capacitance and resistance in vacuum
    xbin=int(fsmax-fsmin)
    freq=[0]*xbin
    Rvac=[0]*xbin
    Cvac=[0]*xbin
    Xvac=[0]*xbin

    La=80.0
    raa=0.2e-3
    for j in range(xbin):
        freq[j]=fsmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        Omega=pi2*freq[j]*1.0e3
        Rvac[j]=80.0*(pi*La/wlen)**2      # vacuum dipole resistance
        k0=Omega/3.0e8
        Xvac[j]=120.0*(np.log(La/(2.0*raa))-1.0)/np.tan(k0*La/2.0)
        Cvac[j]=1.0e12/(Omega*Xvac[j])    # vacuum dipole Capacitance

        #dipole Ca, Xa, and Ra for 3 wires
        Cvac[j]=Cvac[j]*3.0
        Xvac[j]=Xvac[j]
        Rvac[j]=Rvac[j]*1.0e5 #in ohm scaled by 1x10^5 to fit the figure

    print('Min & max vacuum Ca:',("{:8.2f}"*2).format(min(Cvac),max(Cvac)))
    print('Min & max vacuum Ra*10^5:',("{:8.2f}"*2).format(min(Rvac),max(Rvac)))
    N=8   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=80 # for 2D binning not for 1D
    y=np.zeros((2,ybin+1))
    yCX=np.zeros(ybin)
    yRa=np.zeros(ybin)

    xbin=int((fsmax-fsmin)/bwd)
    x=[0]*(xbin+1)
    xfr=[0]*xbin
    for k in range(xbin+1):
        x[k]=fsmin+float(k)*bwd
        if k > 0:
            xfr[k-1]=0.5*(x[k-1]+x[k])

    #--- determine y min & max
    CXmin=min(CaXa)
    CXmax=max(CaXa)
    Ramin=min(Ra)
    Ramax=max(Ra)
    dCX=(CXmax-CXmin)/ybin
    dRa=(Ramax-Ramin)/ybin

    #--- bin size and center of bins for Ca, Ra
    for k in range(ybin+1):
        y[0][k]=CXmin+float(k)*dCX
        y[1][k]=Ramin+float(k)*dRa
        if k > 0:
            yCX[k-1]=0.5*(y[0][k-1]+y[0][k])
            if corx == 'Xa':
                yCX[k-1]=10.0**yCX[k-1]

            yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
            yRa[k-1]=10.0**yRa[k-1]

    # --- calculate number of Ca data points in bins [x, y]
    counts=stats.binned_statistic_2d(fres,CaXa,fres,statistic='count', \
        bins=[x,y[0]])
    a=counts.statistic  # number of data points for Ca
    a=a.transpose()

    rdl=len(a[0])+1-N
    CXb2d=np.zeros((len(a), rdl))
    for k in range(len(a)):
        CXb2d[k]=moving_average.data_points_running_average(a[k], N)
    CXb2d=CXb2d.transpose()

    # --- calculate number of Ra data points in bins [x, y]
    counts=stats.binned_statistic_2d(fres,Ra,fres,statistic='count', \
        bins=[x,y[1]])
    a=counts.statistic  # number of data points for Ra
    a=a.transpose()

    rdl=len(a[0])+1-N
    Rab2d=np.zeros((len(a), rdl))
    for k in range(len(a)):
        Rab2d[k]=moving_average.data_points_running_average(a[k], N)
    Rab2d=Rab2d.transpose()

    # plot number of data points in each bin, onece for yCa (or yRa or ypw)
    # points at given frequency xfr[j]. The data points number is normalized
    # by maximum at each frequency
    for j in range(len(xfr[0:rdl])):
        xx=[xfr[j]]*ybin

        CXbmax=max(CXb2d[j])
        if CXbmax > 0:
            sc=axes[0].scatter(xx,yCX,c=CXb2d[j],cmap=colormp,s=0.1, \
                marker='s')

        Rabmax=max(Rab2d[j])
        if Rabmax > 0:
            sc=axes[1].scatter(xx,yRa,c=Rab2d[j],cmap=colormp,s=0.1, \
                marker='s')

    #----- scaled RPI capacitance
    Ld=41.0     # one brahch length
    rd=0.15e-3
    LR=125.0    # 125 m -branche
    rR=0.20e-3
    fRPI=[32.6,34.0]
    CXRPI=[430e-12,480e-12]
    a1=(np.exp(4.0*pi*eps0*LR/(1.5*CXRPI[0])+1.0)-2.0)*LR*rR*rR
    a2=(np.exp(4.0*pi*eps0*LR/(1.5*CXRPI[1])+1.0)-2.0)*LR*rR*rR
    # one 125 m branch capacitance of RPI = CXRPI * 3/2.
    # and three wires/two branches by another factor 3/2
    CXRPI[0]=3.0/2.0*Ld/LR*3.0/2.0*CXRPI[0]*(np.log(a1/(LR*rR*rR)+2.0)-1.0) \
             /(np.log(a1/(Ld*rd*rd)+2.0)-1.0)
    CXRPI[1]=3.0/2.0*Ld/LR*3.0/2.0*CXRPI[1]*(np.log(a2/(LR*rR*rR)+2.0)-1.0) \
             /(np.log(a2/(Ld*rd*rd)+2.0)-1.0)
    #----- completed scaling RPI capacitance
    RaRPI=[250,500]

    #----- scale Tu et al [2008] reactance & resistance to DSX
    LT=50.0 #one branch length
    rT=0.2
    NT=500.0e6
    Nd=2000.0e6
    fTu=[30.0]
    Omega=pi*fTu[0]*1.0e3
    CXTu=[3486.2/2.0] # one branch reactance
    a1=(np.exp(4.0*pi*Omega*eps0*LT*CXTu[0]+1.0)-2.0)*LT*rT*rT*NT
    #scaled to DSX reactance for two branches/3 wires in Ohm
    CXTu[0]=2.0/3.0*CXTu[0]*LT/Ld*(np.log(a1/(Nd*Ld*rd*rd)+2.0)-1.0) \
            /(np.log(a1/(NT*LT*rT*rT)+2.0)-1.0)
    RaTu=[0]
    RaTu[0]=CXTu[0]/np.tan(88.59*pi/180.0)
    #----- completed scaling Tu et al reactance & resistance

    fChe=[2.55,2.75,3.05,3.20,3.50,3.92,4.36,4.85,5.42,5.90,6.5,7.0,7.5,8.0, \
            8.5,9.0,9.5,10.0]
    CXWang=[0]*18
    CXChe=[0]*18
    for j in range(18):
        CXWang[j]=3e3*(0.7*fChe[j]+0.8557)
        CXChe[j]=3e3*(0.73*fChe[j]+0.8557)
    RaChe=[1490.0,1080,808.0,725.0,655.0,610.0,575.0, \
            550.0,520.0,510.0,505.0,500.0,500.0,500.0, \
            500.0,500.0,500.0,500.0]
    RaWang=[1850.0,1150,828.0,735.0,660.0,615.0,580.0, \
            550.0,520.0,510.0,505.0,500.0,500.0,500.0, \
            500.0,500.0,500.0,500.0]

    if corx == 'Ca':
        axes[0].plot(freq, Cvac, ls='dashed', lw=0.3, color='red')
        axes[0].yaxis.set_minor_locator(AutoMinorLocator())
        CXRPI[0]=CXRPI[0]*1.0e12
        CXRPI[1]=CXRPI[1]*1.0e12
        CXTu[0]=1.0e9/(pi2*fTu[0]*1.0e3*CXTu[0])
        for j in range(18):
            CXWang[j]=1.0e12/(pi2*fChe[j]*1.0e3*CXWang[j]/4.0)/5.0e4
            CXChe[j]=1.0e12/(pi2*fChe[j]*1.0e3*CXChe[j]/4.0)/5.0e4
        txtW='Wang & Bell 2x$10^{-5}$'
        txtC='Chevalier et al 2x$10^{-5}$'
    else:
        CXRPI[0]=1.0e-3/(pi2*fRPI[0]*CXRPI[0]) #reactance in kilo Ohm
        CXRPI[1]=1.0e-3/(pi2*fRPI[1]*CXRPI[1])
        txtW='Wang & Bell [1972]'
        txtC='Chevalier et al [2008]'

        XX=[160e3,18e3]
        #axes[0].plot(ff, XX, ls='dashed', lw=0.3, color='black', label='Fitting')

    axes[0].scatter(fm/1e3, Xa_fm, s=2, marker='+',color='black')
    axes[0].scatter(fh/1e3, Xa_fh, s=2, marker='+',color='black')

    axes[0].scatter(fres, CXth, s=0.04, marker='+',color='black', edgecolors=None, \
        label='Song et al [2007]')
    #axes[0].scatter(fRPI,CXRPI, s=0.04, marker='s', color='red', label='IMAGE RPI')
    axes[0].scatter(fTu,CXTu, s=1, marker='^', color='magenta', label='Tu et al [2008]')
    axes[0].plot(fChe,CXWang, lw=0.3, color='black', label=txtW)
    axes[0].scatter(fChe,CXChe, s=0.06, marker='o', color='black', label=txtC)
    axes[0].plot(freq, Xvac, lw=0.3, color='blue', ls='dashed', \
        label='Dipole in vacuum')
  
    #Balmin impedance (perpendicular antenna orientation)
    ff=10.e3
    eps1=1.0-fpe*fpe/(ff*ff*(1.0-fce*fce/ff**2))
    eps3=1.0-fpe*fpe/(ff*ff)

    av=np.sqrt(abs(eps1/eps3))
    a=av*complex(0.0,1.0)
    eps0=8.85e-12

    Omega=2.0*math.pi*ff

    coef=av/(Omega*2.0*math.pi*eps0*eps1*L)

    print(av, coef, eps1)

    # Ra and Xa for a monopole
    Rb=coef*(np.log(2.0*L/raa)-1.0-0.5*np.log(1.0+av*av))
    Xb=-coef*np.arctan(av)
    print('monopole Balmain Ra & Xa: ',Rb, Xb)

    #3 wires connected in parallel 
    Rb=2.0*Rb/3.0    #in kilo Ohm
    Xb=2.0*Xb/3.0    #in kilo Ohm plot -Xb

    #axes[0].scatter([ff/1.0e3,ff/1.0e3], [-Xb*1e4,-Xb*1e4], s=0.04, marker='s', \
    #    color='blue', label=r'Balmain [1964] $\times 10^4$')
      #----- set axis properties for Ca or Xa------
    axes[0].set_xlim(fmin,fmax)
    axes[0].set_xscale(xscale)
    #axes[0].set_ylim(minCX,maxCX)
    axes[0].set_yscale(yscale)
    axes[0].tick_params(axis='both',which='major',length=1.2,width=0.4, \
        pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
        direction='in')
    axes[0].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
        pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
        direction='in')
    axes[0].set_xticklabels([])
    # --- y-axis label
    if corx == 'Ca':
        axes[0].set_ylabel('$C_a$ (pF)',fontsize=3,labelpad=0.7)
    else:
        axes[0].set_ylabel('$-X_a$ (Ohm)',fontsize=3,labelpad=0.7)

    #----- set axis properties for Ra ------
    #note: / operator cannot be applied to list but can be used for numpy array
    #axes[1].scatter(fres, Rath, s=0.04, marker='+',color='black', \
    #    label='Song et al [2007]')
    #axes[1].contour(xfr[0:rdl], yRa, np.log10(Rab2d.transpose()), cmap=colormp)
    #axes[1].scatter(fRPI,RaRPI, s=0.04, marker='s', color='red', label='IMAGE RPI')
   # ---- plot theoretical Ra with given fpe, fce and I0 vs f (Song et al., 2021) ---
    axes[1].scatter(fm/1e3,Ra_fm, marker='+',s=1, color='black')
    axes[1].scatter(fh/1e3,Ra_fh, marker='+',s=0.04, color='black')
    axes[1].scatter(fres,Rath, marker='+',s=0.04, color='black', edgecolors=None, \
        label='Song et al. [2021]')

    axes[1].scatter(fTu,RaTu, s=1, marker='^', color='magenta', label='Tu et al [2008]')
    axes[1].plot(fChe,RaWang, lw=0.3, color='black', label='Wang & Bell [1972]')
    axes[1].scatter(fChe,RaChe, s=0.06, marker='o', color='black', \
        label='Chevalier et al [2008]')
    axes[1].plot(freq, Rvac, lw=0.3, color='blue', ls='dashed', label='Dipole in vacuum')

    axes[1].set_xlim(fmin,fmax)
    axes[1].set_xscale(xscale)
    axes[1].set_yscale('log')
    axes[1].set_ylim(30,7e4)
    axes[1].tick_params(axis='both',which='major',length=1.2,width=0.4, \
        pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
        direction='in')
    axes[1].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
        pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
        direction='in')
    axes[1].set_xticklabels([])

    #axes[1].scatter([ff/1.0e3,ff/1.0e3],[Rb,Rb],s=0.04, marker='s', color='blue', \
    #    label=r'Balmain [1964]')

    if bname == 'All-Bands':
        axes[1].set_xticks([2.5,5,10,20,30,40])
        axes[1].set_xticklabels(['2.5','5','10','20','30','40'])
    axes[1].set_xlabel('Resonance Frequency (kHz)',fontsize=3, labelpad=0.7)
    axes[1].set_ylabel('$R_a$ (Ohm)',fontsize=3,labelpad=0.7)

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' Inside Nominal Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' Outside Plasmasphere'
    else:
        txt=sL1+' All Regions'

    for j in range(2):
        axes[j].legend(loc='upper center',fontsize=2,bbox_to_anchor=(0.78,0.99), \
            frameon=True)

    xloc=0.5
    axes[0].text(xloc,1.13,datemin[0:10]+'$-$'+datemax[0:10],fontsize=3, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(xloc,1.04,txt,fontsize=3,ha='center',va='bottom', \
        transform=axes[0].transAxes)
    axes[0].text(0.3,0.13,'Multiplied by 3x$10^3$',fontsize=3,ha='center',va='bottom', \
        transform=axes[0].transAxes,color='black',rotation=11)
    axes[1].text(0.3,0.05,'Multiplied by $10^5$',fontsize=3,ha='center',va='bottom', \
        transform=axes[1].transAxes,color='blue',rotation=18)

    # ---- color bar ----
    #xloc=0.085
    #axins = inset_axes(axes[1], width='70%', height='8%', loc='lower left', \
    #bbox_to_anchor=(xloc,-0.38,1,1), bbox_transform=axes[1].transAxes)
    #cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.04, aspect=10, \
    #    fraction=0.1, orientation='horizontal',boundaries=None)
    #cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    #cbar.ax.set_xlabel('Number of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    return canv, fig, toolbar
