#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 18 09:52:03 2019

@author: jtu
"""
import os
import numpy as np
import def_classes
import days_of_year
import string

# function to identify if inside or outside the plasmasphere
# using plasmasphere flag from DSX orbit data
def in_or_out_pp(datet, in_pp, t):
    year=int(datet[0:4])
    month=int(datet[5:7])
    day=int(datet[8:10])
    nday=days_of_year.days_of_year(year, month, day)

    nday=year*1000+nday

    inpl=0
    for m in range(len(in_pp)):
        if nday > in_pp[m][0]:
            continue
        elif nday == in_pp[m][0]:
            if t >= in_pp[m][1] and t < in_pp[m][2]:
                inpl=in_pp[m][3]
                break
        else:
            break

    return inpl

# input high power measuremets data
def input_data(fpath, L, bname, in_pp, ioflag):
    datet=''
    t=0.0
    Qp=0.0
    Qm=0.0
    progn=0

    fname=[]
    fnamec=[]

    data_all=[]

    print('Reading high power transmission data ...')

    for rootp, dirn, fn in os.walk(fpath):
        for files in fn:
            if 'Voltage' in files:
                fname.append(os.path.join(rootp,files))
            if 'Current' in files:
                fnamec.append(os.path.join(rootp,files))

    fname.sort()
    fnamec.sort()

    nf=len(fname)

    # check if there are any high power data files exist
    if nf == 0:
        def_classes.print_message('No high data files in directory: ' \
                                  +dirn+' and its subdirectories')
        return data_all

    fname.sort()

    inpflag=ioflag.get()

    for i in range(0,nf):
        sentv =[]
        sentv3=[]
        fq=[]
        Va1=[]
        phi1=[]
        Va2=[]
        phi2=[]
        Va31=[]
        phi31=[]
        Va32=[]
        phi32=[]

        print(fname[i][len(fname[i])-62:len(fname[i])])
        with open(fname[i], 'r') as fid:
            for j in range(0,9):     #remove headers
                fid.readline()

            #read all the other lines from the file
            fl=fid.readlines()
            fid.close()

        if len(fl) < 37:
            print('No measurement records in the file:')
            print(os.path.basename(fname[i])+'. Open and read next file, if any')
            continue

        #loop lines to assing data to lists
        for j in range(len(fl)):
            if '#====' in fl[j]: #measurment starting delimiter
                k=j
                if j > 0 and (sentv[1] == L or bname =='All-Bands'):
                    inpl = in_or_out_pp(datet, in_pp, t)

                    #select data in & out plasmasphere, or just in or just out
                    if inpflag != 'InOut':
                        if inpflag == 'In': #skip if not in plasmasphere
                            if inpl != 1:
                                continue
                        else:
                            if inpl != 2 or inpl != 3: #skip if in the plasmasphere
                                continue

                    #store processed data in the lists of lists
                    data_sub=[datet,t,sentv,sentv3,[Qp,Qm],fq,Va1,Va31,phi1,phi31 ,\
                        Va2,Va32,phi2,phi32,inpl,progn]

                    data_all.append(data_sub)

                    #rest lists as empty
                    sentv =[]
                    sentv3=[]
                    fq=[]
                    Va1=[]
                    phi1=[]
                    Va2=[]
                    phi2=[]
                    Va31=[]
                    phi31=[]
                    Va32=[]
                    phi32=[]

            # end if and start else
            else: #process measurement data and record then store in lists
                a=fl[j].split()

                if j == k+1:
                    datet=a[2]+' '+a[3]
                    t=float(a[3][0:2])*60+float(a[3][3:5])+float(a[3][6:8])/60.0
                elif j == k+2:
                    progn=int(a[2][1:3])
                elif (j >= k+5 and j <= k+7) or (j >= k+10 and j <= k+12):
                    sentv.append(float(a[5]))
                    sentv3.append(float(a[7]))
                elif j == k+15:
                    Qp=float(a[5])
                elif j == k+20:
                    Qm=float(a[5])
                elif j > k+25:
                    fq.append(float(a[0]))
                    Va1.append(float(a[2]))
                    Va31.append(float(a[4]))
                    ind=a[5].find('00-')
                    if ind !=-1:
                        a[5]=a[5][0:ind+2]+'0'+a[5][ind+3:len(a[5])]
                    phi1.append(float(a[5]))
                    ind=a[6].find('00-')
                    if ind !=-1:
                        a[6]=a[6][0:ind+2]+'0'+a[6][ind+3:len(a[6])]
                    phi31.append(float(a[6]))
                    Va2.append(float(a[8]))
                    Va32.append(float(a[10]))
                    ind=a[11].find('00-')
                    if ind !=-1:
                        a[11]=a[11][0:ind+2]+'0'+a[11][ind+3:len(a[11])]
                    phi2.append(float(a[11]))
                    ind=a[12].find('00-')
                    if ind !=-1:
                        a[12]=a[12][0:ind+2]+'0'+a[12][ind+3:len(a[12])]
                    phi32.append(float(a[12]))

             # end else
        # end for loop of number of lines in a file

        #after reading last measurement record in the file
        if sentv[1] == L or bname =='All-Bands':
            inpl = in_or_out_pp(datet, in_pp, t)

            #select data in & out plasmasphere, or just in or just out
            if inpflag != 'InOut':
                if inpflag == 'In': #skip if not in plasmasphere
                    if inpl != 1:
                        continue
                else:
                    if inpl != 2 or inpl != 3: #skip if in the plasmasphere
                        continue

            #store processed data in the lists of lists
            data_sub=[datet,t,sentv,sentv3,[Qp,Qm],fq,Va1,Va31,phi1,phi31 ,\
                Va2,Va32,phi2,phi32,inpl,progn]

            data_all.append(data_sub)
 
        #end if
    #end for loop of number of files

    print('Finished reading high power transmission data')

    return data_all

#input data and time periods when inside the plasmasphere
def input_inside_pp(fpath):
    pos=fpath.rfind('/')
    fpath=fpath[0:pos+1]
    if fpath[len(fpath)-1] == '/':
        fins=fpath+'DSX_orbit/inside_pp.dat'
    else:
        fins=fpath+'/DSX_orbit/inside_pp.dat'

#define an empty list
    in_pp=[]

#--- check if inside plasmasphere flag file exists
    if not os.path.exists(fins.strip()):
        def_classes.print_message('No such file: '+fins)

        return in_pp

    with open(fins, 'r') as fid:
        #remove headers
        fid.readline()

        #read all the other lines from the file
        fl=fid.readlines()

        fid.close()

#--- store date/time when DSX was inside the plasmashere as a list of list
    for i in range(len(fl)):
        a=fl[i].split()

        alist=[int(a[0]), float(a[1]), float(a[2]), int(a[3])]
        in_pp.append(alist)

    return in_pp
